//
//  KGFacebookManager.h
//  BigBrother
//
//  Created by Ernesto Carrión on 7/26/11.
//  Copyright 2011 LJR Software Limited. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "Facebook.h"
#import "KGFacebookLikeView.h"

#define DEFINE_SHARED_INSTANCE_USING_BLOCK(block) \
static dispatch_once_t pred = 0;                \
__strong static id _sharedObject = nil;         \
dispatch_once(&pred, ^{                         \
_sharedObject = block();                        \
});                                             \
return _sharedObject;                           \


@protocol KGFacebookDelegate

/**
 * These methods are mandatory for knowing if the user logs in or our from facebook
 */
@required
-(void)facebookDidLogin;
-(void)facebookDidLogout;

/**
 * These methods are called as a response to a shareOnwall method invocation.
 * Currently if you tap cancel on the share dialog it would go to DidSuccess instead of DidCancel
 * it's a facebook error and we can't do anything about it
 */
@optional
-(void)dialogDidSuccess;
-(void)dialogDidFailWithError:(NSError *)error;
-(void)dialogDidCancel;

/**
 * Called as a response of Posting and Image on facebook
 */
@optional
-(void)imageSuccesfullyPosted;
-(void)imagePostingFailed;

@end

@interface KGFacebookManager : NSObject <FBRequestDelegate, FBDialogDelegate, FBSessionDelegate, UIWebViewDelegate> {
    
    NSArray * permissions_;
    Facebook * facebook_;
    id<KGFacebookDelegate> delegate_;
    
    //Temps
    NSString * name_;
    NSString * httpLink_;
    NSString * description_;
    NSString * caption_;
    NSString * pictureLink_;
    NSString * sourceLink_;
    UIImage * image_;
}

@property(nonatomic, readonly) Facebook * facebook_;
@property(nonatomic, strong) IBOutlet KGFacebookLikeView * likeView;

/**
 * Shared Facebook instance for using across the app
 */
+(KGFacebookManager *) sharedFacebookManager;

/**
 * Check the cookies for seeing if there is an active facebook session
 */
-(BOOL) isFacebookSessionValid;

/**
 * Logs into Facebook using Facebook app or safary
 */
-(void) logIntoFacebook:(id)delegate;

/**
 * Removes the existing Facebook Session
 */
-(void) logOutOfFacebook:(id)delegate;

/**
 * Post a message in your wall using a dialog
 * 
 * @param httpLink The primary link of the dialog
 * @param name The Links name
 * @param caption The Links caption (below the link name)
 * @param description The links description (below the link caption)
 * @param picture An url of a picture to be posted next to the link
 * @param source A source file (*.swf) to be posted next to the link, (If picture and source are setted only source is used)
 */
-(void)postOnWallWithLink:(NSString *)httpLink 
                 linkName:(NSString *)name 
              linkCaption:(NSString *)caption 
          linkDescription:(NSString *)description
                  picture:(NSString *)pictureLink 
                   source:(NSString *)sourceLink 
                  Delegte:(id<KGFacebookDelegate>)delegate;


/**
 * Upload a image to facebook using the app photo album
 * 
 * @param image The image to be posted on the album
 * @param message The message to be posting within the uploaded photo
 */
-(void)postImage:(UIImage *)image WithMessage:(NSString *)message Delegate:(id)delegate;


/**
 * Upload a image to facebook using the app photo album
 * 
 * @param pageID The id(in numbers or in text) of the facebook page to be liked
 * 
 */
-(void)likePage:(NSString *)pageID;

@end

