    //
//  KGFacebookManager.m
//  BigBrother
//
//  Created by Ernesto Carrión on 7/26/11.
//  Copyright 2011 LJR Software Limited. All rights reserved.
//

#import "KGFacebookManager.h"


NSString * const FBAPPID = @"124069227618469";

@interface KGFacebookManager() {
    
    BOOL postingContent;
    BOOL postingPhoto;
    
}

-(void)displayDialogWithLink:(NSString *)httpLink 
                 linkName:(NSString *)name 
              linkCaption:(NSString *)caption 
          linkDescription:(NSString *)description
                  picture:(NSString *)pictureLink 
                   source:(NSString *)sourceLink;

-(void)postImage:(UIImage *)image WithMessage:(NSString *)message;
-(void) cleanDialogStuff;

@end


@implementation KGFacebookManager

@synthesize facebook_, likeView = _likeView;

#pragma mark - CLASS METHODS

+(KGFacebookManager *)sharedFacebookManager 
{    
  DEFINE_SHARED_INSTANCE_USING_BLOCK(^{
    return [[self alloc] init];
  });
}

#pragma mark - PRIVATE METHODS
-(id)init {
    
	self = [super init];
	if (self != nil) {
        
        //Inits Facebook
        permissions_ = [[NSArray arrayWithObjects:@"read_stream", @"publish_stream", @"offline_access",nil] retain];
        facebook_ = [[Facebook alloc] initWithAppId:FBAPPID andDelegate:self];
        
        //Gets session from prefs
        facebook_.accessToken    = [[NSUserDefaults standardUserDefaults] stringForKey:@"AccessToken"];
        facebook_.expirationDate = (NSDate *) [[NSUserDefaults standardUserDefaults] objectForKey:@"ExpirationDate"];
	}
    
	return self;
}


-(BOOL)isFacebookSessionValid {
    
    return facebook_.isSessionValid;
}


-(void)logIntoFacebook:(id)delegate {
    
    delegate_ = delegate;
    [facebook_ authorize:permissions_];
}

-(void)logOutOfFacebook:(id)delegate {
    
    delegate_ = delegate;
    [facebook_ logout];
}


/** 
 * This method tries to share on your wall, if you're not logged in it stores all the values needed
 * Logs you into facebook and then share on your wall
 */
-(void)postOnWallWithLink:(NSString *)httpLink 
                 linkName:(NSString *)name 
              linkCaption:(NSString *)caption 
          linkDescription:(NSString *)description 
                  picture:(NSString *)pictureLink 
                   source:(NSString *)sourceLink 
                  Delegte:(id<KGFacebookDelegate>)delegate {
    
    delegate_ = delegate;
    httpLink_ = [httpLink retain];
    name_ = [name retain];
    caption_ = [caption retain];
    description_ = [description retain];
    pictureLink_ = [pictureLink retain];
    sourceLink_ = [sourceLink retain];
    
    if ([self isFacebookSessionValid]) {
        
        [self displayDialogWithLink:httpLink linkName:name linkCaption:caption linkDescription:description picture:pictureLink source:sourceLink];
        
    } else {
        
        postingContent = YES;
        [self logIntoFacebook:delegate_];
        
    }
    
}


/** 
 * This method tries to upload an image on the app album, if you're not logged in it stores all the values needed
 * Logs you into facebook and then upload the image
 */
-(void)postImage:(UIImage *)image WithMessage:(NSString *)message Delegate:(id)delegate {
    
    delegate_ = delegate;
    image_ = [image retain];
    name_ = [message retain];
    
    if ([self isFacebookSessionValid]) {
        
        [self postImage:image WithMessage:message];
        
    } else {
        
        postingPhoto = YES;
        [self logIntoFacebook:delegate_];
    }
}

/** 
 * This method open an internal WebView that goes to a specific facebook page
 * Then the user is responsable for liking the page and closing the webView
 */
-(void)likePage:(NSString *)pageID {
    
    UINib * nib = [UINib nibWithNibName:@"KGFacebookLikeView" bundle:nil];
    [nib instantiateWithOwner:self options:nil];
    KGFacebookLikeView * likeView = self.likeView;
    self.likeView = nil;
    
    [likeView loadFacebookPage:pageID withDelegate:self];
    
    UIWindow* window = [UIApplication sharedApplication].keyWindow;
    if (!window) {
        window = [[UIApplication sharedApplication].windows objectAtIndex:0];
    }
    
    //likeView.frame = window.frame;
    likeView.frame = CGRectMake(0, 20, 320, 460);
    [likeView animate];
    [window addSubview:likeView];
    
}

/** 
 * Post something in your wall
 */
-(void)displayDialogWithLink:(NSString *)httpLink 
                    linkName:(NSString *)name 
                 linkCaption:(NSString *)caption 
             linkDescription:(NSString *)description 
                     picture:(NSString *)pictureLink 
                      source:(NSString *)sourceLink {
    
    NSString * _link = [[httpLink copy] autorelease];
    NSString * _name = [[name copy] autorelease];
    NSString * _caption = [[caption copy] autorelease];
    NSString * _description = [[description copy] autorelease];
    NSString * _pictureLink = [[pictureLink copy] autorelease];
    NSString * _sourceLink = [[sourceLink copy] autorelease];
    
    if (!_link)
        _link = @"";
    
    if (!_name)
        _name = @"";
    
    if (!_caption)
        _caption = @"";
    
    if (!_description)
        _description = @"";
    
    if (!_pictureLink)
        _pictureLink = @"";
    
    if (!_sourceLink)
        _sourceLink = @"";
    
    
    NSMutableDictionary* params = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                   _link, @"link",
                                   _caption, @"caption",
                                   _name, @"name",
                                   _description, @"description",
                                   _pictureLink, @"picture",
                                   _sourceLink, @"source", nil];
    
    [facebook_ dialog:@"feed" andParams:params andDelegate:self];
    
}


/** 
 * Uploads some image to the app album
 */
-(void)postImage:(UIImage *)image WithMessage:(NSString *)message {
        
    UIImage * _image = [[UIImage alloc] initWithCGImage:image.CGImage];
    NSString * _message = [[message copy] autorelease];
    
    if (!_image)
        _image = [[[UIImage alloc] init] autorelease];
    
    if (!_message)
        _message = @"";
    
    NSMutableDictionary *params = [NSMutableDictionary dictionaryWithObjectsAndKeys:
                                   _image, @"picture",
                                   _message, @"message", nil];
    
    [facebook_ requestWithGraphPath:@"me/photos"
                          andParams:params
                      andHttpMethod:@"POST"
                        andDelegate:self];
}

/** 
 * Cleans all the stored data, the data is stored when you try to do soemthing but you're not logged in
 * when 'something' is done, then the data is not needed anymore
 */
-(void)cleanDialogStuff {
    
    
    postingPhoto = postingContent = NO;
    
    [name_ release];
    name_ = nil;
    
    [httpLink_ release];
    httpLink_ = nil;
    
    [caption_ release];
    caption_ = nil;
    
    [description_ release];
    description_ = nil;
    
    [pictureLink_ release];
    pictureLink_ = nil;
    
    [sourceLink_ release];
    description_ = nil;
    
    [image_ release];
    image_ = nil;
}

#pragma mark - FB DELEGATE

/** 
 * Stores FB Credentials when a succesfull login, if the login was produced because the user wanted to share or post an image
 * then we should complete that action with the stored data
 */
-(void)fbDidLogin {
    
    NSUserDefaults * prefs = [NSUserDefaults standardUserDefaults];
    [prefs setObject:facebook_.accessToken forKey:@"AccessToken"];
    [prefs setObject:facebook_.expirationDate forKey:@"ExpirationDate"];
    [prefs synchronize];
    
    [delegate_ facebookDidLogin];
    
    if (postingContent) {
        
        [self displayDialogWithLink:httpLink_ 
                           linkName:name_ 
                        linkCaption:caption_
                    linkDescription:description_
                            picture:pictureLink_
                             source:sourceLink_];
    }
    
    if (postingPhoto) {
        
        [self postImage:image_ 
            WithMessage:name_];
    }    
}

/** 
 * Deletes fb credentials
 */
-(void) fbDidLogout {
    
    NSUserDefaults * prefs = [NSUserDefaults standardUserDefaults];
    [prefs removeObjectForKey:@"AccessToken"];
    [prefs removeObjectForKey:@"ExpirationDate"];
    [prefs synchronize];
    
    [delegate_ facebookDidLogout];
    
    if (postingPhoto || postingPhoto) {
        
        [self cleanDialogStuff];
    }
    
}

/** 
 * If the user did not login and the login was triggered because of the user intention of posting or sharing something, then we need to inform the user about the failing status
 */
-(void)fbDidNotLogin:(BOOL)cancelled {
    
    NSUserDefaults * prefs = [NSUserDefaults standardUserDefaults];
    [prefs removeObjectForKey:@"AccessToken"];
    [prefs removeObjectForKey:@"ExpirationDate"];
    [prefs synchronize];
    
    [self cleanDialogStuff];
    
    if (postingPhoto) {
        
        if ([(NSObject *)delegate_ respondsToSelector:(@selector(imagePostingFailed))]) {
            [delegate_ imagePostingFailed];
        }
    }
    
    if (postingContent) {
        
        if ([(NSObject *)delegate_ respondsToSelector:(@selector(dialogDidFailWithError:))]) {
            [delegate_ dialogDidFailWithError:nil];
        }
    }
}

-(void)fbDidExtendToken:(NSString *)accessToken expiresAt:(NSDate *)expiresAt {
    
    NSUserDefaults * prefs = [NSUserDefaults standardUserDefaults];
    [prefs setObject:accessToken forKey:@"AccessToken"];
    [prefs setObject:expiresAt forKey:@"ExpirationDate"];
    [prefs synchronize];
}

/** 
 * If the facebook session was invalidated, we need to try and log the user again
 */
-(void)fbSessionInvalidated {
        
    puts("invalidated");
    [self logIntoFacebook:delegate_];
}



//TODO: Refactor this - Need to understand this problem more deeply
/**
 * This is called when the share dialog fails
 * If it fails for an OAuth method we should try to log the user in again
 * If it fails for the -999 error, we don't know what to do, currently we are retrying the requested action
 */
-(void)dialog:(FBDialog *)dialog didFailWithError:(NSError *)error {

    NSString * message = [[error userInfo] valueForKeyPath:@"error.message"];
    
    if ([message hasPrefix:@"Error validating access token"]) { //Oauth error
        
        [self logIntoFacebook:delegate_];
        
    } else { //other error
        
        /*
         This is the super hack
         
         Basically i am getting a -999 error, which means:
         
         According to "Foundation Constants Reference", error code -999 means "NSURLErrorCancelled".
         
         Description:
         
         Returned when an asynchronous load is canceled. A Web Kit framework delegate will receive this error when it performs a cancel operation on a loading resource. Note that an NSURLConnection or NSURLDownload delegate will not receive this error if the download is canceled. Available in iOS 2.0 and later. Declared in NSURLError.h.
         
         What i do is once i get this error i reques to post it again and then it works.
         */
        
        if ([error code] == -999) {
            
            [self postOnWallWithLink:httpLink_ linkName:name_ linkCaption:caption_ linkDescription:description_ picture:pictureLink_ source:sourceLink_ Delegte:delegate_];
            return;
        }
        
        if ([(NSObject *)delegate_ respondsToSelector:(@selector(dialogDidFail))]) {
            [delegate_ dialogDidFailWithError:error];
        }
        
        [self cleanDialogStuff];
    }

}

-(void)dialogDidComplete:(FBDialog *)dialog {
        
    if ([(NSObject *)delegate_ respondsToSelector:(@selector(dialogDidSuccess))]) {
        [delegate_ dialogDidSuccess];
    }
    
    [self cleanDialogStuff];
}

-(void)dialogDidNotComplete:(FBDialog *)dialog {
    
    if ([(NSObject *)delegate_ respondsToSelector:(@selector(dialogDidCancel))]) {
        [delegate_ dialogDidCancel];
    }
    
    [self cleanDialogStuff];
}

/**
 * This is called when the uploading image fails
 * If it fails for an OAuth method we should try to log the user in again
 */
-(void)request:(FBRequest *)request didFailWithError:(NSError *)error {
    
    NSString * message = [[error userInfo] valueForKeyPath:@"error.message"];
    
    if ([message hasPrefix:@"Error validating access token"]) { //Oauth error
        
        [self logIntoFacebook:delegate_];
        
    } else { //other error
        
        if ([(NSObject *)delegate_ respondsToSelector:(@selector(imagePostingFailed))]) {
            [delegate_ imagePostingFailed];
        }
            
        [self cleanDialogStuff];
        puts("error");
    }
    
}

-(void)request:(FBRequest *)request didLoad:(id)result {
    
    if ([(NSObject *)delegate_ respondsToSelector:(@selector(imageSuccesfullyPosted))]) {
        [delegate_ imageSuccesfullyPosted];
    }
    
    [self cleanDialogStuff];
    puts("exito");
}



#pragma mark - WebView Delegate

-(void)webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error {
    
    //Further improvement
}

-(void)webViewDidFinishLoad:(UIWebView *)webView {

    //Further improvement
    
}

-(void)webViewDidStartLoad:(UIWebView *)webView {
    
    //Further improvement
}

-(BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType {
    
    //Further improvement
    return YES;
}


#pragma mark - DEFAULT

-(void)dealloc {
    
    [permissions_ release];
    [facebook_ release];
    
    [image_ release];
    [httpLink_ release];
    [name_ release];
    [caption_ release];
    [description_ release];
    [pictureLink_ release];
    [sourceLink_ release];
    
    [super dealloc];
}

@end
